@@ -18,8 +18,8 @@
 
 ;; You should have received a copy of the GNU General Public License
 ;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+;; Boston, MA 02110-1301, USA.
 
 ;;; Commentary:
 
@@ -376,7 +376,8 @@ Add an entry here when adding a new search engine.")
 ;;; User Customizable Variables:
 
 (defgroup nnir nil
-  "Search nnmh and nnml groups in Gnus with Glimpse, freeWAIS-sf, or EWS.")
+  "Search nnmh and nnml groups in Gnus with Glimpse, freeWAIS-sf, or EWS."
+  :group 'gnus)
 
 ;; Mail backend.
 
@@ -531,18 +532,31 @@ that it is for swish++, not Glimpse."
   :type '(regexp)
   :group 'nnir)
 
-;; Swish-E.  Next three variables Copyright (C) 2000 Christoph Conrad
-;; <christoph.conrad@gmx.de>.
+;; Swish-E.
 ;; URL: http://sunsite.berkeley.edu/SWISH-E/
 ;; New version: http://www.boe.es/swish-e
+;; Variables `nnir-swish-e-index-file', `nnir-swish-e-program' and
+;; `nnir-swish-e-additional-switches'
+;; Copyright (C) 2000 Christoph Conrad <christoph.conrad@gmx.de>.
 
+(make-obsolete-variable 'nnir-swish-e-index-file
+			'nnir-swish-e-index-files)
 (defcustom nnir-swish-e-index-file
   (expand-file-name "~/Mail/index.swish-e")
   "*Index file for swish-e.
-This could be a server parameter."
+This could be a server parameter.
+It is never consulted once `nnir-swish-e-index-files', which should be
+used instead, has been customized."
   :type '(file)
   :group 'nnir)
 
+(defcustom nnir-swish-e-index-files
+  (list nnir-swish-e-index-file)
+  "*List of index files for swish-e.
+This could be a server parameter."
+  :type '(repeat (file))
+  :group 'nnir)
+
 (defcustom nnir-swish-e-program "swish-e"
   "*Name of swish-e search executable.
 This cannot be a server parameter."
@@ -1195,10 +1209,10 @@ Tested with swish-e-2.0.1 on Windows NT 4.0."
       (erase-buffer)
 
       (message "Doing swish-e query %s..." query)
-      (let* ((index-file
+      (let* ((index-files
 	      (or (nnir-read-server-parm
-		   'nnir-swish-e-index-file server)
-		  (error "Missing parameter `nnir-swish-e-index-file'")))
+		   'nnir-swish-e-index-files server)
+		  (error "Missing parameter `nnir-swish-e-index-files'")))
 	     (additional-switches
 	      (nnir-read-server-parm
 	       'nnir-swish-e-additional-switches server))
@@ -1206,7 +1220,7 @@ Tested with swish-e-2.0.1 on Windows NT 4.0."
 			nil		; input from /dev/null
 			t		; output
 			nil		; don't redisplay
-			"-f" ,index-file
+			"-f" ,@index-files
 			,@additional-switches
 			"-w"
 			,qstring	; the query, in swish-e format
